@@ -2,10 +2,10 @@
 # Copyright (c) 2022 MBition GmbH
 
 from dataclasses import dataclass
-from typing import Optional, List
+from typing import Optional, List, Dict, Any
 
 from .utils import read_description_from_odx
-from .odxlink import OdxLinkId, OdxDocFragment
+from .odxlink import OdxLinkId, OdxDocFragment, OdxLinkDatabase
 
 @dataclass()
 class State:
@@ -17,6 +17,11 @@ class State:
     long_name: Optional[str] = None
     description: Optional[str] = None
 
+    def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
+        return { self.odx_id: self }
+
+    def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
+        pass
 
 def read_state_from_odx(et_element, doc_frags: List[OdxDocFragment]):
     short_name = et_element.findtext("SHORT-NAME")
