@@ -2,9 +2,9 @@
 # Copyright (c) 2022 MBition GmbH
 
 from dataclasses import dataclass
-from typing import Optional, List
+from typing import Optional, List, Dict, Any
 
-from .odxlink import OdxLinkId, OdxDocFragment
+from .odxlink import OdxLinkId, OdxDocFragment, OdxLinkDatabase
 
 @dataclass()
 class StateTransition:
@@ -17,6 +17,11 @@ class StateTransition:
     source_short_name: Optional[str] = None
     target_short_name: Optional[str] = None
 
+    def _build_odxlinks(self) -> Dict[OdxLinkId, Any]:
+        return { self.odx_id: self }
+
+    def _resolve_references(self, odxlinks: OdxLinkDatabase) -> None:
+        pass
 
 def read_state_transition_from_odx(et_element, doc_frags: List[OdxDocFragment]):
     short_name = et_element.findtext("SHORT-NAME")
